@@ -10,6 +10,8 @@ except DistributionNotFound:
 finally:
     del get_distribution, DistributionNotFound
 
+# This is not used here but exported to __init__.py
+from mau.parsers.base_parser import ParserError
 from mau.parsers.main_parser import MainParser
 from mau.parsers.nodes import DocumentNode, ContainerNode
 from mau.visitors.html_visitor import HTMLVisitor
@@ -17,12 +19,17 @@ from mau.visitors.asciidoctor_visitor import AsciidoctorVisitor
 from mau.visitors.markua_visitor import MarkuaVisitor
 
 
+class ConfigurationError(ValueError):
+    """ Used to signal an error in the configuration """
+
+
 class Mau:
     def __init__(
         self,
         config,
         target_format,
         default_templates=None,
+        custom_templates=None,
         templates_directory=None,
         full_document=False,
     ):
@@ -30,12 +37,12 @@ class Mau:
         # will be used here and also injected into the variables
         self.config = config
 
-        # The target format that corresponds to a specific visitor
-        self.target_format = target_format
-
         # A dictionary with the default templates.
         self.default_templates = default_templates
 
+        # A dictionary with the custom templates.
+        self.custom_templates = custom_templates
+
         # A directory that contains the custom templates as files
         self.templates_directory = templates_directory
 
@@ -48,9 +55,6 @@ class Mau:
         self.variables = {}
 
     def process(self, text):
-        # Update the config with the target format
-        self.config["target_format"] = self.target_format
-
         # Store the config under the Mau namespace
         self.variables["mau"] = self.config
 
@@ -67,18 +71,16 @@ class Mau:
         output = wrapper_node_class(parser.nodes)
 
         # Select the visitor class
-        if self.target_format == "html":
+        target_format = self.config.get("target_format", "html")
+
+        if target_format == "html":
             visitor_class = HTMLVisitor
-        elif self.target_format == "asciidoctor":
+        elif target_format == "asciidoctor":
             visitor_class = AsciidoctorVisitor
-        elif self.target_format == "markua":
+        elif target_format == "markua":
             visitor_class = MarkuaVisitor
-
-        # Retrieve the TOC
-        toc = [i.asdict() for i in parser.toc]
-
-        # Retrieve the footnotes
-        footnotes = [i.asdict() for i in parser.footnotes]
+        else:
+            raise ConfigurationError(f"Target format {target_format} is not available")
 
         # Initialise the visitor
         # Use the parser variables so that the visitor
@@ -86,10 +88,11 @@ class Mau:
         # variables defined inside the text
         visitor = visitor_class(
             default_templates=self.default_templates,
+            custom_templates=self.custom_templates,
             templates_directory=self.templates_directory,
             config=parser.variables,
-            toc=toc,
-            footnotes=footnotes,
+            toc=parser.toc,
+            footnotes=parser.footnotes,
         )
 
         # Replace variables with the ones processed by the parser
