@@ -1,24 +1,42 @@
+import copy
+
 from pygments import highlight
 from pygments.lexers import get_lexer_by_name
 from pygments.formatters import get_formatter_by_name
 
+from mau.parsers import nodes
 from mau.visitors.visitor import Visitor
 
+
 DEFAULT_TEMPLATES = {
-    "admonition.html": (
-        '<div class="admonition {{ class }}">'
-        '<i class="{{ icon }}"></i>'
+    "block-admonition.html": (
+        '<div class="admonition {{ kwargs.class }}">'
+        '<i class="{{ kwargs.icon }}"></i>'
         '<div class="content">'
-        '<div class="title">{{ label }}</div>'
+        '<div class="title">{{ kwargs.label }}</div>'
         "<div>{{ content }}</div>"
         "</div></div>"
     ),
     "block.html": (
-        '<div{% if type %} class="{{ type }}"{% endif %}>'
+        '<div class="{{ blocktype }}{% if classes %} {{ classes }}{% endif %}">'
         '{% if title %}<div class="title">{{ title }}</div>{% endif %}'
         '<div class="content">{{ content }}</div>'
         "</div>"
     ),
+    "block-source.html": (
+        '<div{% if blocktype %} class="{{ blocktype }}"{% endif %}>'
+        '{% if title %}<div class="title">{{ title }}</div>{% endif %}'
+        '<div class="content">{{ content }}</div>'
+        '{% if kwargs.callouts %}<div class="callouts">'
+        "<table><tbody>"
+        "{% for callout in kwargs.callouts %}<tr>"
+        "<td>{{ callout[0] }}</td>"
+        "<td>{{ callout[1] }}</td>"
+        "</tr>{% endfor %}"
+        "</tbody></table>"
+        "</div>{% endif %}"
+        "</div>"
+    ),
     "callout.html": '<span class="callout">{{ name }}</span>',
     "class.html": '<span class="{{ classes }}">{{ content }}</span>',
     "command.html": "{{ content }}",
@@ -26,7 +44,7 @@ DEFAULT_TEMPLATES = {
     "document.html": "<html><head></head><body>{{ content }}</body></html>",
     "footnote_def.html": (
         '<div id="{{ defanchor }}">'
-        '<a href="#{{ refanchor }}">{{ number }}</a> {{ text }}</div>'
+        '<a href="#{{ refanchor }}">{{ number }}</a> {{ content }}</div>'
     ),
     "footnote_ref.html": (
         "<sup>"
@@ -39,7 +57,7 @@ DEFAULT_TEMPLATES = {
     "image.html": (
         '<div class="imageblock">'
         '<div class="content">'
-        '<img src="{{ uri }}"{% if alt_text %} alt="{{ alt_text }}"{% endif %}>'
+        '<img src="{{ uri }}"{% if alt_text %} alt="{{ alt_text }}"{% endif %} />'
         '{% if title %}<div class="title">{{ title }}</div>{% endif %}'
         "</div></div>"
     ),
@@ -57,31 +75,20 @@ DEFAULT_TEMPLATES = {
     "list_item.html": "<li>{{ content }}</li>",
     "macro.html": "",
     "paragraph.html": "<p>{{ content }}</p>",
-    "quote.html": (
-        "<blockquote>" "{{ content }}" "<cite>{{ attribution }}</cite>" "</blockquote>"
+    "block-quote.html": (
+        "<blockquote>"
+        "{{ content }}"
+        "<cite>{% if kwargs.attribution %}{{ kwargs.attribution }}{% else %}{{ secondary_content }}{% endif %}</cite>"
+        "</blockquote>"
     ),
     "raw.html": "{{ content }}",
     "sentence.html": "{{ content }}",
-    "source.html": (
-        '<div class="code">'
-        '{% if title %}<div class="title">{{ title }}</div>{% endif %}'
-        '<div class="content">{{ code }}</div>'
-        '{% if callouts %}<div class="callouts">'
-        "<table><tbody>"
-        "{% for callout in callouts %}<tr>"
-        "<td>{{ callout[0] }}</td>"
-        "<td>{{ callout[1] }}</td>"
-        "</tr>{% endfor %}"
-        "</tbody></table>"
-        "</div>{% endif %}"
-        "</div>"
-    ),
     "star.html": "<strong>{{ content }}</strong>",
     "text.html": "{{ value }}",
     "toc.html": "<div>{% if entries%}<ul>{{ entries }}</ul>{% endif %}</div>",
     "toc_entry.html": (
         "<li>"
-        '<a href="#{{ anchor }}">{{ text }}</a>'
+        '<a href="#{{ header.anchor }}">{{ header.value }}</a>'
         "{% if children %}<ul>{{ children }}</ul>{% endif %}"
         "</li>"
     ),
@@ -95,14 +102,22 @@ class HTMLVisitor(Visitor):
 
     def __init__(
         self,
-        default_templates=DEFAULT_TEMPLATES,
+        default_templates=None,
+        custom_templates=None,
         templates_directory=None,
         config=None,
         toc=None,
         footnotes=None,
     ):
+        default_templates = (
+            copy.deepcopy(default_templates)
+            if default_templates is not None
+            else copy.deepcopy(DEFAULT_TEMPLATES)
+        )
+
         super().__init__(
             default_templates=default_templates,
+            custom_templates=custom_templates,
             templates_directory=templates_directory,
             config=config,
             toc=toc,
@@ -110,53 +125,17 @@ class HTMLVisitor(Visitor):
         )
 
     def _visit_class(self, node):
-        return {
-            "classes": " ".join(node["classes"]),
-            "content": self.visit(node["content"]),
-        }
-
-    def _visit_link(self, node):
-        return {"text": node["text"], "target": node["target"]}
-
-    def _visit_header(self, node):
-        return {
-            "level": node["level"],
-            "value": node["value"],
-            "anchor": node["anchor"],
-        }
-
-    def _visit_quote(self, node):
-        return {
-            "attribution": node["attribution"],
-            "content": "".join(self.visitlist(node["content"])),
-        }
-
-    def _visit_raw(self, node):
-        return {"content": "\n".join(self.visitlist(node["content"]))}
+        node["classes"] = " ".join(node["classes"])
+        return super()._visit_class(node)
 
     def _visit_admonition(self, node):
-        return {
-            "node_types": [
-                f'admonition_{node["class"]}',
-                "admonition",
-            ],
-            "class": node["class"],
-            "icon": node["icon"],
-            "label": node["label"],
-            "content": "".join(self.visitlist(node["content"])),
-        }
+        node["node_types"] = [f'admonition_{node["class"]}']
+        self._reducelist(node, ["content"], join_with="")
+        return node
 
-    def _visit_block(self, node):
-        return {
-            "type": node["blocktype"],
-            "content": "".join(self.visitlist(node["content"])),
-            "title": self.visit(node["title"]),
-            "kwargs": node["kwargs"],
-        }
-
-    def _visit_source(self, node):
+    def _visit_block_engine_source(self, node):
         # The Pygments lexer for the given language
-        lexer = get_lexer_by_name(node["language"])
+        lexer = get_lexer_by_name(node["kwargs"]["language"])
 
         # Fetch global configuration for Pygments and for the HtmlFormatter
         pygments_config = self.config.get("pygments", {})
@@ -180,7 +159,7 @@ class HTMLVisitor(Visitor):
         hl_lines = [i for i in hl_lines if i != ""]
 
         # Pygments starts counting form 1, Mau from 0
-        highlight_markers = [i + 1 for i in node["highlights"]]
+        highlight_markers = [i + 1 for i in node["kwargs"]["highlights"]]
 
         # Merge the two
         hl_lines = list(set(hl_lines) | set(highlight_markers))
@@ -192,7 +171,7 @@ class HTMLVisitor(Visitor):
         formatter = get_formatter_by_name("html", **formatter_config)
 
         # Get the raw source from the node
-        src = "\n".join([i["value"] for i in node["code"]])
+        src = node["content"]
 
         # Highlight the source with Pygments
         highlighted_src = highlight(src, lexer, formatter)
@@ -201,10 +180,9 @@ class HTMLVisitor(Visitor):
         highlighted_src_lines = highlighted_src.split("\n")
 
         # Inject callout markers in the highlighted code
-        callout_markers = node["callouts"]["markers"]
-        callout_contents = node["callouts"]["contents"]
+        callout_markers = node["kwargs"]["callouts"]["markers"]
+        callout_contents = node["kwargs"]["callouts"]["contents"]
 
-        callouts = {}
         for linenum, callout_name in callout_markers.items():
             highlighted_src_lines[linenum] = "{line} {callout}".format(
                 line=highlighted_src_lines[linenum],
@@ -218,75 +196,33 @@ class HTMLVisitor(Visitor):
             for name, text in callout_contents.items()
         ]
 
-        return {
-            "code": highlighted_src,
-            "language": node["language"],
-            "callouts": callouts_list,
-            "title": self.visit(node["title"]),
-        }
+        node["content"] = highlighted_src
+        node["kwargs"]["callouts"] = callouts_list
 
-    def _visit_document(self, node):
-        return {
-            "content": "".join([self.visit(item) for item in node["content"]]),
-        }
+        return node
 
-    def _visit_list_item(self, node):
-        return {"content": self.visit(node["content"])}
+    def _visit_toc_entry(self, node, exclude_tags=None):
+        exclude_tags = exclude_tags or []
 
-    def _visit_list(self, node):
-        return {
-            "ordered": node["ordered"],
-            "items": "".join([self.visit(i) for i in node["items"]]),
-        }
+        tags_excluded = [i for i in node["header"]["tags"] if i in exclude_tags]
 
-    def _visit_toc_entry(self, node):
-        return {
-            "anchor": node["anchor"],
-            "text": node["text"],
-            "children": "".join([self.visit(i) for i in node["children"]]),
-        }
+        if len(tags_excluded) > 0:
+            # Return an empty text node
+            return nodes.TextNode(value="").asdict()
 
-    def visit_toc(self, nodes):
-        keys = {"entries": "".join([self.visit(i) for i in nodes])}
-        return self._render("toc", **keys)
+        self._reducelist(node, ["children"], join_with="", exclude_tags=exclude_tags)
+        return node
 
-    def _visit_footnote_ref(self, node):
-        return {
-            "number": node["number"],
-            "refanchor": node["refanchor"],
-            "defanchor": node["defanchor"],
-        }
+    def _visit_command_toc(self, node):
+        exclude_tags = node["kwargs"].get("exclude_tags", [])
+        toc_node = self.toc.asdict()
 
-    def _visit_footnote_def(self, node):
-        return {
-            "number": node["number"],
-            "refanchor": node["refanchor"],
-            "defanchor": node["defanchor"],
-            "text": "".join([self.visit(i) for i in node["content"]]),
-        }
+        self._reducelist(toc_node, ["entries"], join_with="", exclude_tags=exclude_tags)
 
-    def visit_footnotes(self, nodes):
-        keys = {"entries": "".join([self.visit(i) for i in nodes])}
-        return self._render("footnotes", **keys)
+        return toc_node
 
-    def _visit_command(self, node):
-        if node["name"] == "toc":
-            return {"content": "".join(self.visit_toc(self.toc))}
+    def _visit_command_footnotes(self, node):
+        footnotes_node = self.footnotes.asdict()
+        self._reducelist(footnotes_node, ["entries"], join_with="")
 
-        if node["name"] == "footnotes":
-            return {"content": "".join(self.visit_footnotes(self.footnotes))}
-
-    def _visit_content_image(self, node):
-        return {
-            "node_types": ["image"],
-            "uri": node["uri"],
-            "title": self.visit(node["title"]),
-            "alt_text": node["alt_text"],
-        }
-
-    def _visit_image(self, node):
-        return {
-            "node_types": ["inline_image"],
-            "uri": node["uri"],
-            "alt_text": node["alt_text"],
-        }
+        return footnotes_node
