@@ -1,6 +1,9 @@
+import pytest
+
 from unittest.mock import patch
 
 from mau.parsers.nodes import DocumentNode, ContainerNode
+from mau.parsers.base_parser import ParserError
 from mau.parsers.main_parser import MainParser
 from mau.visitors.html_visitor import HTMLVisitor
 
@@ -93,7 +96,7 @@ def test_style_verbatim():
     _test(source, expected)
 
 
-def test_class():
+def test_class_old_style():
     source = "[classname]#text with that class#"
 
     expected = ['<p><span class="classname">text with that class</span></p>']
@@ -101,6 +104,14 @@ def test_class():
     _test(source, expected)
 
 
+def test_class_new_style():
+    source = '[class]("text with classes", "class1,class2")'
+
+    expected = ['<p><span class="class1 class2">text with classes</span></p>']
+
+    _test(source, expected)
+
+
 def test_link():
     source = '[link](https://somedomain.org/the/path,"link text")'
 
@@ -152,6 +163,31 @@ def test_quote():
     _test(source, expected)
 
 
+def test_quote_attribution_in_secondary_content_supports_styles():
+    source = dedent(
+        """
+        [quote]
+        ----
+        Learn about the Force, Luke.
+        ----
+        _Star Wars_, 1977
+        """
+    )
+
+    expected = [
+        remove_indentation(
+            """
+            <blockquote>
+              <p>Learn about the Force, Luke.</p>
+              <cite><p><em>Star Wars</em>, 1977</p></cite>
+            </blockquote>
+            """
+        )
+    ]
+
+    _test(source, expected)
+
+
 def test_admonition():
     source = dedent(
         """
@@ -181,23 +217,6 @@ def test_admonition():
     _test(source, expected)
 
 
-def test_raw():
-    source = dedent(
-        """
-        [raw]
-        ----
-        <div class="someclass">
-          <p>This is HTML</>
-        </div>
-        ----
-        """
-    )
-
-    expected = ["""<div class="someclass">\n  <p>This is HTML</>\n</div>"""]
-
-    _test(source, expected)
-
-
 @patch("mau.parsers.main_parser.header_anchor")
 def test_block(header_anchor_mock):
     header_anchor_mock.return_value = "XXXXXX"
@@ -215,16 +234,7 @@ def test_block(header_anchor_mock):
     )
 
     expected = [
-        remove_indentation(
-            """
-            <div class="sometype">
-              <div class="content">
-                <h1 id="XXXXXX">A block</h1>
-                <p>This contains headers, paragraphs and blocks</p>
-              </div>
-            </div>
-            """
-        )
+        """<div class="sometype"><div class="content"><h1 id="XXXXXX">A block</h1>\n<p>This contains headers, paragraphs and blocks</p></div></div>"""
     ]
 
     _test(source, expected)
@@ -273,17 +283,7 @@ def test_basic_block_title(header_anchor_mock):
     )
 
     expected = [
-        remove_indentation(
-            """
-            <div class="sometype">
-              <div class="title">Title</div>
-              <div class="content">
-                <h1 id="XXXXXX">A block</h1>
-                <p>This contains headers, paragraphs and blocks</p>
-              </div>
-            </div>
-            """
-        )
+        """<div class="sometype"><div class="title">Title</div><div class="content"><h1 id="XXXXXX">A block</h1>\n<p>This contains headers, paragraphs and blocks</p></div></div>"""
     ]
 
     _test(source, expected)
@@ -432,7 +432,7 @@ def test_include_image():
             """
             <div class="imageblock">
               <div class="content">
-                <img src="/path/to/it.jpg">
+                <img src="/path/to/it.jpg" />
               </div>
             </div>
             """
@@ -455,7 +455,7 @@ def test_include_image_alt_text():
             """
             <div class="imageblock">
               <div class="content">
-                <img src="/path/to/it.jpg" alt="This is a beautiful image">
+                <img src="/path/to/it.jpg" alt="This is a beautiful image" />
               </div>
             </div>
             """
@@ -478,7 +478,7 @@ def test_include_image_title():
             """
             <div class="imageblock">
               <div class="content">
-                <img src="/path/to/it.jpg">
+                <img src="/path/to/it.jpg" />
                 <div class="title">A nice <em>caption</em></div>
               </div>
             </div>
@@ -495,3 +495,229 @@ def test_macro():
     expected = ["<p></p>"]
 
     _test(source, expected)
+
+
+def test_macro_template():
+    v = HTMLVisitor()
+    v.default_templates["macro.html"] = "whatever"
+
+    ast = init_ast("[unknown](This is an unknown macro)")
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["<p>whatever</p>"]
+
+
+def test_macro_uses_specific_template():
+    v = HTMLVisitor()
+    v.default_templates["macro.html"] = "whatever"
+    v.default_templates["macro-unknown.html"] = "the right one"
+
+    ast = init_ast("[unknown](This is an unknown macro)")
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["<p>the right one</p>"]
+
+
+def test_macro_provides_args_and_kwargs():
+    v = HTMLVisitor()
+    v.default_templates["macro.html"] = "whatever"
+    v.default_templates["macro-unknown.html"] = "{{ args[0] }} - {{ kwargs.name}}"
+
+    ast = init_ast('[unknown]("some text", name=mau)')
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["<p>some text - mau</p>"]
+
+
+def test_macro_alias_value_for_first_unnamed_arg():
+    v = HTMLVisitor()
+    v.default_templates["macro.html"] = "whatever"
+    v.default_templates["macro-unknown.html"] = "{{ value }} - {{ kwargs.name}}"
+
+    ast = init_ast('[unknown]("some text", name=mau)')
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["<p>some text - mau</p>"]
+
+
+def test_block_engine_raw():
+    source = dedent(
+        """
+        [block, engine=raw]
+        ----
+        <div class="someclass">
+          <p>This is HTML</>
+        </div>
+        ----
+        """
+    )
+
+    expected = [
+        """<div class="block"><div class="content"><div class="someclass">\n  <p>This is HTML</>\n</div></div></div>"""
+    ]
+
+    _test(source, expected)
+
+
+def test_block_engine_default():
+    source = dedent(
+        """
+        = Header out
+
+        [block, engine=default]
+        ----
+        = Header in
+
+        ::toc:
+        ----
+        """
+    )
+
+    parser = init_parser(source)
+    parser.parse()
+
+    result = visitlist([node.asdict() for node in parser.nodes], toc=parser.toc)
+
+    assert result == [
+        '<h1 id="header-out">Header out</h1>',
+        '<div class="block"><div class="content"><h1 id="header-in">Header in</h1>\n<div><ul><li><a href="#header-out">Header out</a></li><li><a href="#header-in">Header in</a></li></ul></div></div></div>',
+    ]
+
+
+def test_block_engine_mau():
+    source = dedent(
+        """
+        = Header out
+
+        [block, engine=mau]
+        ----
+        = Header in
+
+        ::toc:
+        ----
+        """
+    )
+
+    parser = init_parser(source)
+    parser.parse()
+
+    result = visitlist([node.asdict() for node in parser.nodes], toc=parser.toc)
+
+    assert result == [
+        '<h1 id="header-out">Header out</h1>',
+        '<div class="block"><div class="content"><h1 id="header-in">Header in</h1><div><ul><li><a href="#header-in">Header in</a></li></ul></div></div></div>',
+    ]
+
+
+def test_block_template():
+    v = HTMLVisitor()
+    v.default_templates["block.html"] = "whatever"
+
+    ast = init_ast(
+        dedent(
+            """
+            [block]
+            ----
+            Primary content
+            ----
+            Secondary content
+            """
+        )
+    )
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["whatever"]
+
+
+def test_block_uses_specific_template():
+    v = HTMLVisitor()
+    v.default_templates["block.html"] = "whatever"
+    v.default_templates["block-unknown.html"] = "the right one"
+
+    ast = init_ast(
+        dedent(
+            """
+            [unknown]
+            ----
+            Primary content
+            ----
+            Secondary content
+            """
+        )
+    )
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["the right one"]
+
+
+def test_block_supports_kwargs():
+    v = HTMLVisitor()
+    v.default_templates["block.html"] = "whatever"
+    v.default_templates[
+        "block-unknown.html"
+    ] = "{{ kwargs.name }} - {{ kwargs.colour }}"
+
+    ast = init_ast(
+        dedent(
+            """
+            [unknown, name=myblock, colour=green]
+            ----
+            ----
+            """
+        )
+    )
+
+    output = [v.visit(node) for node in ast]
+
+    assert output == ["myblock - green"]
+
+
+def test_block_does_not_support_args():
+    v = HTMLVisitor()
+    v.default_templates["block.html"] = "whatever"
+    v.default_templates["block-unknown.html"] = ""
+
+    with pytest.raises(ParserError):
+        ast = init_ast(
+            dedent(
+                """
+                [unknown, value1, value2]
+                ----
+                Primary content
+                ----
+                Secondary content
+                """
+            )
+        )
+
+
+@patch("mau.parsers.main_parser.header_anchor")
+def test_command_consumes_args(header_anchor_mock):
+    header_anchor_mock.return_value = "XXXXXX"
+
+    source = dedent(
+        """
+        ::unknown:unused1,unuse2=value2
+
+        [sometype]
+        ++++
+        = A block
+
+        This contains headers, paragraphs and blocks
+
+        ++++
+        """
+    )
+
+    expected = [
+        "",
+        """<div class="sometype"><div class="content"><h1 id="XXXXXX">A block</h1>\n<p>This contains headers, paragraphs and blocks</p></div></div>""",
+    ]
+
+    _test(source, expected)
